/*
 * Header for glue functions to switch between C and generated code.
 *
 * Copyright (C) 2005 Nick Tarleton.
 * Licensed under the GNU General Public License; see COPYING.txt.
 */

#ifndef _multi_GLUE_H
#define _multi_GLUE_H

#include <asmtypes.h>
#include "xlate.h"

// Z80 register set
typedef union {
	struct __attribute__((packed)) {
		WORD bc, de, hl, a, ix, iy;
		WORD sp;
	} words;
	struct __attribute__((packed)) {
		// there are no endianness worries as this structure will only
		// be used on the calc
		BYTE b, c, d, e, h, l;
		BYTE __f_does_not_exist, a;
		BYTE ixh, ixl, iyh, iyl;
		// sp not byte-addressable
	} bytes;
} Z80_REGS;

/* Calls the generated code at gen_code, setting the Z80 'registers' to regs.
 * regs will be updated with the new values, except for sp.
 */
void call_gen(M68K_CODE gen_code, Z80_REGS* regs);

/* Convenient no-op function. Just returns. */
void no_op(void);

/* Executes a Z80 RET, handling the stack properly. */
extern void z80_return;

#endif // _multi_GLUE_H
