/*
 * Implementation of assorted global functions.
 *
 * Copyright (C) 2006 Nick Tarleton.
 * Licensed under the GNU General Public License; see COPYING.txt.
 */

#include <tigcclib.h>

#include "misc.h"
#include "target.h"

// global variables
BYTE* z80_base;
DWORD add_ghost_space;
TARGET* cur_target;

/* Reports an error and exits. Uses printf() format. */
void report_error(const char* format, ...)
{
        char buf[256]; /* hopefully enough */
        va_list args;
	cleanup();
        va_start(args, format);
        vsprintf(buf, format, args);
	va_end(args);
        DlgMessage("Error", buf, BT_OK, BT_NONE);
	/* it's unsafe to exit() out of an error frame, so: */
	ER_throw(ER_EXIT);
}

static void** remember_list = NULL;
static int remember_nr = 0, remember_alloc = 0;
#define REMEMBER_GROW 10

void* malloc_remember(size_t size)
{
	void* ptr = malloc_throw(size);
	if(remember_nr + 1 > remember_alloc) {
		remember_alloc += REMEMBER_GROW;
		remember_list = realloc_throw(remember_list,
			remember_alloc * sizeof(void*));
	}
	remember_list[remember_nr] = ptr;
	remember_nr++;

	return ptr;
}

void* realloc_remember(void* ptr, size_t size)
{
	int prev_loc = 0, i;

	if(ptr == NULL)
		return malloc_remember(size);

	for(i = 0; i < remember_nr; i++) {
		if(remember_list[i] == ptr) {
			prev_loc = i;
			break;
		}
	}
	if(prev_loc == 0)
		report_error("realloc_remember passed %p not in list (size %lu)", ptr, size);

	ptr = realloc_throw(ptr, size);
	remember_list[prev_loc] = ptr;

	return ptr;
}

void free_remembered(void)
{
	int i;
	if(remember_list == NULL) // unlikely but theoretically possible...
		return;

	for(i = 0; i < remember_nr; i++)
		free(remember_list[i]);

	free(remember_list);
	remember_list = NULL; // reset statics for next run
	remember_nr = remember_alloc = 0;
}
