/*
 * Sundry globals.
 *
 * Copyright (C) 2005 Nick Tarleton.
 * Licensed under the GNU General Public License; see COPYING.txt.
 */

#ifndef _multi_MISC_H
#define _multi_MISC_H

#ifdef TESTING_ON_PC
#	include <stdlib.h>
#	include "compat.h"
#	define CALLBACK
#else
#	include <tigcclib.h>
#endif

/* base address of Z80 space
 * 
 * NOTE: this is Z80 address 0, but the ROM is not emulated! This points to
 * random memory. Only touch addresses after z80_base + 0x8000.
 */
extern BYTE* z80_base;

/* ghots space offset to add to generated code addreses
 * set to 0x40000 on HW1 (unneeded but no harm) and HW2 (needed)
 * set t0 0 on HW3 (no ghost space)
 */
extern DWORD add_ghost_space;

/* Reports an error and exits. Uses printf() format. */
void report_error(const char* format, ...)
	__attribute__ ((noreturn, format(printf, 1, 2)));

#ifndef TESTING_ON_PC
/* Allocates some memory and remembers to free it on program exit.
 * Uses malloc_throw().
 */
void* malloc_remember(size_t size);

/* Same thing, for realloc_throw(). */
void* realloc_remember(void* ptr, size_t size);

/* atexit() callback to free memory allocated with malloc_remember(). */
void free_remembered(void);

/* cleanup code (defined in main.c) */
CALLBACK void cleanup(void);

#endif

#endif // _multi_MISC_H
