/*
 * Test framework for code generation.
 * Requires ttdasm installed.
 *
 * Copyright (C) 2005 Nick Tarleton.
 * Licensed under the GNU General Public License; see COPYING.txt.
 */

#include <stdio.h>
#include <stdlib.h>

#include "compat.h"
#include "globals.inc"

#include "xlate.h"
#include "68k_gen.h"

typedef struct BLOCK {
	Z80_LOC z80_start; // start of Z80 code
	M68K_CODE m68k_code; // location of 68K code
	struct BLOCK* next; // next block in translated list
	struct { // trampoline to jump to block code
		WORD jmp_instr __attribute__((packed));
		DWORD target __attribute__((packed));
	} trampoline __attribute__((aligned(2)));
} BLOCK;

int main(void)
{
	FILE* output = fopen("test-asm-out", "wb");
	XLATE_STATE state;
	BLOCK block;

	cur_state = &state;
	state.block = &block;
	state.out_pos = state.alloc = 0;
	state.next = NULL;
	block.m68k_code = NULL;
	block.next = NULL;

	// Put the emit_* calls to test here.
	emit_bin_op_imm_ea_Q(SIZE_W, ADDQ, 1, MODE_pAn_disp8_Rn, 0);
	emit_pAn_disp8_Rn(2, RDATA, RWORD, 0);
	emit_bin_op_imm_ea_Q(SIZE_W, ADDQ, 1, MODE_pAn_disp8_Rn, 0);
	emit_pAn_disp8_Rn(2, RDATA, RLONG, 0);
	emit_bin_op_imm_ea_Q(SIZE_W, ADDQ, 1, MODE_pAn_disp8_Rn, 0);
	emit_pAn_disp8_Rn(2, RADDR, RWORD, 0);
	emit_bin_op_imm_ea_Q(SIZE_W, ADDQ, 1, MODE_pAn_disp8_Rn, 0);
	emit_pAn_disp8_Rn(2, RADDR, RLONG, 0);

	for(; state.out_pos > 0; block.m68k_code++, state.out_pos--)
		putc(*block.m68k_code, output);

	fclose(output);
	system("ttdasm -raw test-asm-out");
	remove("test-asm-out");
	return 0;
}
