| vim:filetype=asm68k:smartindent:foldmethod=marker:foldmarker={{{,}}}
| TI-83+ keyboard input routines.
|
| Copyright (C) 2006 Nick Tarleton.
| Licensed under the GNU General Public License; see COPYING.txt.

.include "asm.inc"
.include "ti83plus.inc"

.text
.even

| return CSC in a, but only let a single keypress count, but make arrow keys
| and DEL repeat
.globl TI83P_func_getcsc
TI83P_func_getcsc: |{{{
	movem.l %d0-%d2/%d4-%d5,-(%sp)
	bsr	TI83P_internal_read_keys
	movem.l	(%sp)+,%d0-%d2/%d4-%d5
	move.b	%d7,%d3
	lea	key_was_pressed_last_time(%pc),%a2
	tst.b	%d3
	bne	1f
	| no key was pressed
	clr.b	(%a2)
	rts
1:	| key was pressed
	cmp.b	#4,%d3
	bls	3f	| return immediately if arrow (XXX inexact, should wait)
	cmp.b	#skDel,%d3
	beq	3f	| or if DEL
	tst.b	(%a2)
	beq	2f
	| still pressed from last time
	clr.b	%d3
	rts
	| pressed for first time
2:	addq.b	#1,(%a2)
3:	rts
|}}}

.globl key_was_pressed_last_time
key_was_pressed_last_time:
	.space 1
	.even

| save the last bitmask output
.globl TI83P_out_kbd
TI83P_out_kbd: |{{{
	lea	lastmask(%pc),%a2
	move.b	%d3,(%a2)
	rts
|}}}

| return output keys in a
| FIXME: seeing as how I don't know what the correct behavior is when multiple
| rows are unmasked, I haven't even tried to emulate it
.globl TI83P_in_kbd
TI83P_in_kbd: |{{{
	movem.l	%d0-%d5,-(%sp)
	bsr	TI83P_internal_read_keys
	movem.l	(%sp)+,%d0-%d5

	lea	lastmask(%pc),%a2
	move.b	(%a2),%d6
	moveq	#0,%d3	| zero a
	lea	keybits(%pc),%a2
	moveq	#6,%d7	| counter
1:
	lsr.b	#1,%d6	| take 1 bit from mask
	bcc	2f	| skip if bit is 1
	addq.l	#1,%a2	| next row
	dbra	%d7,1b
	rts		| no bits unmasked
2:	move.b	(%a2),%d3 | get value
	not.b	%d3	| invert
	rts
|}}}

| stores bitmasks for pressed keys
.even
keybits:
	.space 7
| next value to read from port 1
lastmask:
	.space 1
.even

| read the keyboard, fill keybits, return csc in d7
| destroys d0-6, a2-4
TI83P_internal_read_keys: |{{{
	lea	keybits(%pc),%a2
	clr.l	(%a2)+
	clr.w	(%a2)+
	clr.b	(%a2)
	subq.l	#6,%a2
	lea	TI83P_CSC_table(%pc),%a3
	lea	TI83P_key_table(%pc),%a4
	move.b	#0xfe,%d0	| d0 = mask = 0b11111110
	moveq	#0,%d4
	moveq	#0,%d7
1:
	move.b	%d0,(0x600019)	| select row
	moveq	#24,%d2	| delay
2:
	dbra	%d2,2b
	move.b	(0x60001b),%d2	| read bits
	moveq	#0,%d1	| d1 = column to read
2:
	btst	%d1,%d2
	bne	3f
	| key is pressed!
	move.b	(%a3),%d7	| CSC
	move.b	(%a4),%d4	| bitmask
	move.b	%d4,%d5		| unpack
	lsr.b	#4,%d4		| d4 = row
	moveq	#0xf,%d6
	and.b	%d6,%d5		| d5 = col
	bset	%d5,(%d4,%a2)	| set bit in keybits
3:
	addq.l	#1,%a3	| next CSC
	addq.l	#1,%a4	| next keybits
	addq.b	#1,%d1	| next bit
	moveq	#7,%d6
	cmp.b	%d6,%d1
	bls	2b
	| done with a row
	rol.b	#1,%d0	| rotate mask
	bcs	1b	| if the 0 hasn't rotated around, do next row
	| done, phew!
	rts
|}}}

| tables are in (row,col) order for the 89 keyboard matrix
| table of _getCSC values
TI83P_CSC_table: |{{{
	.byte skUp,skLeft,skDown,skRight,sk2nd,skMode,skAlpha,skGraphvar
	.byte skEnter,skAdd,skSub,skMul,skDiv,skPower,skClear,skGraph
	.byte skChs,sk3,sk6,sk9,skRParen,skTan,skVars,skTrace
	.byte skDecPnt,sk2,sk5,sk8,skLParen,skCos,skPrgm,skZoom
	.byte sk0,sk1,sk4,sk7,skComma,skSin,skMatrix,skWindow
	.byte skStat,skStore,skLn,skLog,skSquare,skRecip,skMath,skYEqu
	.byte skDel,0,0,0,0,0,0,0,0
|}}}

| table of values for port 1
| high nibble is row, low nibble is col
TI83P_key_table: |{{{
	.byte 0x03,0x01,0x00,0x02,0x65,0x66,0x57,0x47
	.byte 0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x60
	.byte 0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x61
	.byte 0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x62
	.byte 0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x63
	.byte 0x37,0x51,0x52,0x53,0x54,0x55,0x56,0x64
	.byte 0x67,0x07,0x07,0x07,0x07,0x07,0x07,0x07 | (0,7) is not used
|}}}
