| vim:filetype=asm68k:smartindent:foldmethod=marker:foldmarker={{{,}}}
| TI-83+ target: library function implementations for text functions.
|
| Copyright (C) 2006 Nick Tarleton.
| Licensed under the GNU General Public License; see COPYING.txt.

.include "asm.inc"
.include "ti83plus.inc"
.include "ion.inc"

.text
.even

| curcol = currow = 0
.globl TI83P_func_homeup
TI83P_func_homeup: |{{{
	clr.b	currow-0x8000(%a1)
	clr.b	curcol-0x8000(%a1)
	rts
|}}}

| new line
.globl TI83P_func_newline
TI83P_func_newline: |{{{
	bclr	#textScrolled,textflags(%a0,%d5.l)
	clr.b	curcol-0x8000(%a1)
	move.b	currow-0x8000(%a1),%d7
	addq.b	#1,%d7
	cmp.b	#8,%d7
	bne	1f
	|	scroll screen
	btst	#appAutoScroll,appflags(%a0,%d5.l) | FIXME I don't know if this
	beq	2f | is correct behavior: just return to the beginning of line 7
	movem.l	%d0-%d2/%a0-%a1,-(%sp)
	move.l	#((8 << 16) + A_REVERSE),-(%sp)
	pea	viewport_rect(%pc)
	pea	viewport_rect(%pc)
	movea.l	0xC8,%a0
	movea.l	4*ScrRectScroll(%a0),%a0
	jsr	(%a0)
	lea	12(%sp),%sp
	movem.l	(%sp)+,%d0-%d2/%a0-%a1
	bset	#textScrolled,textflags(%a0,%d5.l)
2:	moveq #7,%d7
1:	move.b	%d7,currow-0x8000(%a1)
	rts

| rectangle for scrolling
viewport_rect:
	.byte	VIEWPORT_X,VIEWPORT_Y
	.byte	VIEWPORT_X+VIEWPORT_W,VIEWPORT_Y+VIEWPORT_H

|}}}

| put large character at cursor position on viewport
| input: d6.b = character
| destroys d0-4, d6-7, a2-3
TI83P_internal_putchar_large: |{{{
	moveq	#0,%d0
	move.b	curcol-0x8000(%a1),%d0
	mulu.w	#6,%d0	| x = 6 * curcol
	moveq	#0,%d1
	move.b	currow-0x8000(%a1),%d1
	lsl.w	#3,%d1	| y = 8 * currow
	moveq	#(SCREEN_W/8),%d3
	lea	VIEWPORT_START,%a2
|}}}

| put large character at a pixel position in some buffer
| input: d0.w = x, d1.w = y, d3.w = buffer_w, d6.b = character,
| 	a2 = buffer
| destroys d0-4, d6-7, a2-3
TI83P_internal_vputchar_large: |{{{
	movem.l	%a0-%a1,-(%sp)
	| setup registers for multi_Sprite8
	| d0 = x, d1 = y, d2 = h, d3 = screen_w_in_bytes
	| a2 = buff, a3 = sprite
	| d4 = mode
	moveq	#8,%d2	| height

	btst	#textInverse,textflags(%a0,%d5.l) | check textinverse
	bne	1f
	lea	blank_sprite(%pc),%a3 | non-inverse: mask with blank sprite
	moveq	#SPRT_AND,%d4
	bra	2f
1:	lea	inverse_sprite(%pc),%a3 | inverse: put solid sprite for inversion
	moveq	#SPRT_OR,%d4
2:
	movem.l	%d0-%d1,-(%sp) | save params
	bsr	multi_Sprite8
	movem.l	(%sp)+,%d0-%d1 | restore

	cmp.b	#0xf1,%d6 | last character in font
	sls.b	%d7 | all others have the same glyph as char 0
	and.b	%d7,%d6
1:	mulu.w	#7,%d6	| d6 *= 7 - index of character in large_font
	lea	TI83P_large_font(%pc),%a3 | sprite
	adda.w	%d6,%a3
	moveq	#7,%d2 | height
	moveq	#SPRT_XOR,%d4 | mode
	bsr	multi_Sprite8 | put character
	movem.l	(%sp)+,%a0-%a1
	rts

| blank sprite for masking
blank_sprite:
	.byte	3,3,3,3,3,3,3,3
| inverse sprite for inverse text
inverse_sprite:
	.byte	0xfc,0xfc,0xfc,0xfc,0xfc,0xfc,0xfc,0xfc
|}}}

| put large character
| input: a = character
.globl TI83P_func_putmap
TI83P_func_putmap: |{{{
	movem.l	%d0-%d4,-(%sp)	| save regs
	moveq	#0,%d6
	move.b	%d3,%d6
	bsr	TI83P_internal_putchar_large
	movem.l	(%sp)+,%d0-%d4
	rts
|}}}

| put large character and advance cursor
| input: a = character
.globl TI83P_func_putc
TI83P_func_putc: |{{{
	bsr	TI83P_func_putmap
	| the falling off is intentional!
|}}}

| advance cursor, destroys d7
TI83P_internal_advance_cursor: |{{{
	move.b	curcol-0x8000(%a1),%d7 | advance cursor
	addq.b	#1,%d7
	move.b	%d7,curcol-0x8000(%a1)
	cmp.b	#16,%d7 | new line?
	beq	TI83P_func_newline
	rts
|}}}

hl_format_str:
	.asciz	"%5hu"	| unsigned short padded to 5 with spaces
	.even

| display hl in large font, 5 characters, space-padded
.globl TI83P_func_disphl
TI83P_func_disphl: |{{{
	| first do sprintf
	movem.l	%d0-%d2/%a0-%a1,-(%sp)
	move.w	%d2,-(%sp)
	pea	hl_format_str(%pc)	| format
	pea	op1-0x8000(%a1)		| buf
	movea.l	0xC8,%a0
	movea.l	4*sprintf(%a0),%a0
	jsr	(%a0)
	lea	10(%sp),%sp
	movem.l	(%sp)+,%d0-%d2/%a0-%a1
	move.w	#op1,%d2
	| fall off to puts
|}}}

| put string
| input: hl = string
| output: hl = byte after null
.globl TI83P_func_puts 
TI83P_func_puts: |{{{
	movem.l	%d0-%d4,-(%sp)	| save regs
	lea	(%a0,%d2.l),%a4 | load string to a4
1:	moveq	#0,%d6
	move.b	(%a4)+,%d6
	beq	2f	| exit on null terminator
	moveq	#-0x2A,%d7 | 0xD6
	cmp.b	%d7,%d6 | newline
	beq	3f
	bsr	TI83P_internal_putchar_large | put character
	bsr	TI83P_internal_advance_cursor
	bra	1b

2:	movem.l	(%sp)+,%d0-%d4
	suba.l	%a0,%a4 | reload hl
	move.w	%a4,%d2
	rts

3:	bsr	TI83P_func_newline
	bra	1b
|}}}

| same thing for *Pascal* string (first byte is length)
.globl TI83P_func_putps
TI83P_func_putps: |{{{
	movem.l	%d0-%d4,-(%sp)	| save regs
	lea	(%a0,%d2.l),%a4 | load string to a4
	move.b	(%a4)+,%d7
	addq.b	#1,%d7
1:	subq.b	#1,%d7
	beq	2f | exit on 0
	moveq	#0,%d6
	move.b	(%a4)+,%d6
	moveq	#-0x2A,%d0 | 0xD6
	cmp.b	%d0,%d6 | newline
	beq	3f
	move.w	%d7,-(%sp)
	bsr	TI83P_internal_putchar_large | put character
	bsr	TI83P_internal_advance_cursor
	move.w	(%sp)+,%d7
	bra	1b

2:	movem.l	(%sp)+,%d0-%d4
	suba.l	%a0,%a4 | reload hl
	move.w	%a4,%d2
	rts

3:	bsr	TI83P_func_newline
	bra	1b
|}}}

| get small char width
| input: d6.b = character
| output: d7.b = width
| destroys a3
TI83P_internal_small_char_width: |{{{
	moveq	#-0x20,%d7 | = 0xe0
	cmp.b	%d7,%d6 | ensure character not too large
	bls	1f
	moveq	#0,%d7
	rts
1:	lea	TI83P_small_char_widths(%pc),%a3
	ror.l	#1,%d6	| save LSb
	move.b	(%a3,%d6.w),%d7
	rol.l	#1,%d6	| restore LSb and put in CF
	bcs	1f	| carry set = odd = no need to shift
	lsr.b	#4,%d7
1:	and.b	#0xf,%d7
	rts
|}}}

| put small character at a pixel position in some buffer
| input: d0.w = x, d1.w = y, d3.w = buffer_w, d6.b = character,
| 	a2 = buffer
| destroys d0-4, d6-7, a2-3
TI83P_internal_vputchar_small: |{{{
	bsr	TI83P_internal_small_char_width
	tst.b	%d7
	bne	1f | ensure nonzero
	rts
1:
	movem.l %a0-%a1,-(%sp)
	| prepare mask
	| for character of width 3, will look like:
	| 0001 1111 for normal
	| 1110 0000 for inverse
	move.b	#0x80,%d2 | 1000 0000
	asr.b	%d7,%d2	  | 1111 0000
	add.b	%d2,%d2	  | 1110 0000
	btst	#textInverse,textflags(%a0,%d5.l)
	beq	1f
	moveq	#SPRT_OR,%d4	| inverse
	bra	2f
1:	moveq	#SPRT_AND,%d4	| normal
	not.b	%d2
2:
	move.w	#5,%d7
	subq.l	#6,%sp	| create on stack
	movea.l	%sp,%a3
1:	move.b	%d2,(%a3)+
	dbra	%d7,1b
	subq.l	#6,%a3

	| setup registers for multi_Sprite8
	| d0 = x, d1 = y, d2 = h, d3 = screen_w_in_bytes
	| a2 = buff, a3 = sprite
	| d4 = mode
	movem.l	%d0-%d1,-(%sp) | save params
	moveq	#6,%d2 | height
	bsr	multi_Sprite8
	movem.l	(%sp)+,%d0-%d1 | restore
	addq.l	#6,%sp	| pop mask

	move.w	%d6,%d7	| d6 *= 5 - index of character in small_font
	add.w	%d7,%d7
	add.w	%d7,%d7
	add.w	%d7,%d6
	lea	TI83P_small_font(%pc),%a3 | sprite
	adda.w	%d6,%a3
	moveq	#5,%d2 | height
	moveq	#SPRT_XOR,%d4 | mode
	bsr	multi_Sprite8 | put character
	movem.l	(%sp)+,%a0-%a1
	rts
|}}}

| put small or large character in %d6 (depending on fontflags.fracdrawlfont)
| at pen position
| on buffer or screen (depending on sgrflags.textwrite)
| and advance pen
TI83P_vputmap_internal: |{{{
	moveq	#0,%d0
	move.b	pencol-0x8000(%a1),%d0 | x
	moveq	#0,%d1
	move.b	penrow-0x8000(%a1),%d1 | y

	btst	#textwrite,sgrflags(%a0,%d5.l)
	beq	1f
	lea	gbuf-0x8000(%a1),%a2 | buffer
	moveq	#(VIEWPORT_W/8),%d3
	bra	2f
1:	lea	VIEWPORT_START,%a2 | screen
	moveq	#(SCREEN_W/8),%d3
2:
	btst	#fracdrawlfont,fontflags(%a0,%d5.l)
	beq	1f
	bsr	TI83P_internal_vputchar_large | large
	addq.b	#6,pencol-0x8000(%a1)
	bra	2f
1:	move.l	%d6,-(%sp)
	bsr	TI83P_internal_vputchar_small | small
	move.l	(%sp)+,%d6
	bsr	TI83P_internal_small_char_width
	add.b	%d7,pencol-0x8000(%a1)

2:	rts
|}}}

| put small or large character in a (depending on fontflags.fracdrawlfont)
| at pen position
| on buffer or screen (depending on sgrflags.textwrite)
| and advance pen
.globl TI83P_func_vputmap |{{{
TI83P_func_vputmap:
	movem.l	%d0-%d4,-(%sp)	| save regs
	move.l	%d3,%d6 | character
	bsr	TI83P_vputmap_internal
	movem.l	(%sp)+,%d0-%d4
	rts
|}}}

| same thing for string at (hl)
.globl TI83P_func_vputs |{{{
TI83P_func_vputs:
	movem.l	%d0-%d4,-(%sp)	| save regs
	lea	(%a0,%d2.l),%a4 | load string to a4
1:	moveq	#0,%d6
	move.b	(%a4)+,%d6
	beq	2f	| exit on null terminator
	bsr	TI83P_vputmap_internal | put character
	bra	1b

2:	movem.l	(%sp)+,%d0-%d4
	suba.l	%a0,%a4 | reload hl
	move.w	%a4,%d2
	rts
|}}}

| same thing with length in b
.globl TI83P_func_vputsn |{{{
TI83P_func_vputsn:
	movem.l	%d0-%d4,-(%sp)	| save regs
	lsr.w	#8,%d0	| move b to low byte
	lea	(%a0,%d2.l),%a4 | load string to a4
	addq.b	#1,%d0
1:	subq.b	#1,%d0
	beq	2f | exit on b=0
	moveq	#0,%d6
	move.b	(%a4)+,%d6
	move.w	%d0,-(%sp)
	bsr	TI83P_vputmap_internal | put character
	move.w	(%sp)+,%d0
	bra	1b

2:	movem.l	(%sp)+,%d0-%d4
	suba.l	%a0,%a4 | reload hl
	move.w	%a4,%d2
	rts
|}}}
