/*
 * Translation core.
 *
 * Copyright (C) 2005 Nick Tarleton.
 * Licensed under the GNU General Public License; see COPYING.txt.
 */

#ifndef _multi_XLATE_H
#define _multi_XLATE_H

#include "misc.h"

// endianness conversion
extern inline WORD swap_w(WORD n)
{
	return ((n & 0xFF) << 8) | (n >> 8);
}
extern inline DWORD swap_l(DWORD n)
{
	return ((n & 0xFF) << 24) | ((n & 0xFF00) << 8) | ((n & 0xFF0000) >> 8) | (n >> 24);
}

#ifdef LITTLE_ENDIAN_PC
#	define from_LE_w(x) (x)
#	define from_LE_l(x) (x)
#	define to_BE_w(x) swap_w(x)
#	define to_BE_l(x) swap_l(x)
#else // calculator or BE PC
#	define from_LE_w(x) swap_w(x)
#	define from_LE_l(x) swap_l(x)
#	define to_BE_w(x) (x)
#	define to_BE_l(x) (x)
#endif

BYTE read_b(void);
WORD read_w(void);

/* convenience types */
typedef WORD Z80_LOC;
typedef BYTE* M68K_CODE;

/* Translate some code. The core function.
 * Returns the address of the translated code.
 * If the code in question is already translated, returns the already-translated
 * copy.
 */
M68K_CODE translate(Z80_LOC loc);

/* Call this function at the start of the program to zero static variables. */
void xlate_zero_statics(void);

typedef struct BLOCK {
	Z80_LOC z80_start; // start of Z80 code
	M68K_CODE m68k_code; // location of 68K code
	struct BLOCK* next; // next block in translated list
	struct { // trampoline to jump to block code
		WORD jmp_instr __attribute__((packed));
		DWORD target __attribute__((packed));
	} trampoline __attribute__((aligned(2)));
} BLOCK;

/* Translation state. Used to allow nested translations. */
typedef struct XLATE_STATE {
	BLOCK* block;
	WORD in_pos;
	int out_pos, alloc;
	char reg_swapped[6];
	char ccr_saved;
	struct XLATE_STATE* next;
} XLATE_STATE;

extern XLATE_STATE* cur_state;

#endif
